﻿using System;

using Bluefin.Data;
using Bluefin.Web.Client;

namespace Bluefin_Dotnet_Guide
{
    /// <summary>
    /// This class includes code and data to run basic transaction tests.
    /// </summary>
    public class TransactionsTester
    {
        /// <summary>
        /// Gets last transaction id or null, if operation failed or didn't return transaction id
        /// </summary>
        public String TransactionID
        {
            get;
            protected set;
        }

        /// <summary>
        /// Gests last transaction amount or null, if operation failed or didn't return transaction amount
        /// </summary>
        public BluefinAmount TransactionAmount
        {
            get;
            protected set;
        }

        /// <summary>
        /// Gets last transaction tender type or null, if operation failed or didn't return tender_type
        /// </summary>
        public TenderType TransactionTenderType
        {
            get;
            protected set;
        }

        static void Main(string[] args)
        {
            TransactionsTester tester = new TransactionsTester();
            tester.RunAllTests();
        }

        /// <summary>
        /// Run basic transaction related tests.
        /// </summary>
        public void RunAllTests()
        {
            Boolean isSuccess =
                CardSale()
                && CheckSale()
                && CheckCredit()
                && SwipedCardSale()
                && CardCapture()
                && CardRefund()
                && CheckRefund()
                && AuthorizeNotOnFileCard()
                && AuthorizeCardOnFile()
                && StoreCard()
                && UpdateStoredCard()
                && SafeCommunicationCardSale()
                && SafeCommunicationCheckSale()
            ;

            Console.WriteLine("\nAll Tests Succeded: {0}", isSuccess);
        }

        /// <summary>
        /// Runs card sale transaction.
        /// </summary>
        /// <returns></returns>
        public Boolean CardSale()
        {
            Console.WriteLine("CardSale()");
            return
                CardSale(TransactionsHelper.SampleCardTransaction())
                    .Success();
        }

        /// <summary>
        /// Runs sale transaction for check.
        /// </summary>
        public Boolean CheckSale()
        {
            Console.WriteLine("CheckSale()");
            return
                CheckSale(TransactionsHelper.SampleCheckTransaction())
                    .Success();
        }

        public Boolean CheckCredit()
        {
            Console.WriteLine("CheckCredit()");
            if (!CheckSale())
                return false;

            CardTransactionRequest ctr = new CardTransactionRequest();

            ctr.TokenID = this.TransactionID;
            ctr.Amount  = this.TransactionAmount; // credit on full amount

            TransactionResponse response = BluefinGateway.Transaction.Credit(ctr);

            return
                response.Success();
        }

        /// <summary>
        /// Runs sale transaction for swiped card.
        /// </summary>
        public Boolean SwipedCardSale()
        {
            string cardSwipes = @"%B4167239876543211^GIFT/CARD^1604121100000001000000943000000?;4167239876543211=1604121100000001943?";
            Console.WriteLine("SwipedCardSale()");
            return
                SwipedCardSale(TransactionsHelper.SampleSwipedCardTransaction(cardSwipes))
                    .Success();
        }

        /// <summary>
        /// Authorize charge on card. If succesfull, capture funds.
        /// </summary>
        public Boolean CardCapture()
        {
            Console.WriteLine("CardCapture()");
            if (!AuthorizeNotOnFileCard())
                return false;

            // Capture full amount: CTOR without amount
            CaptureTransactionRequest ctr = new CaptureTransactionRequest(this.TransactionID);
            return 
                Capture(ctr).Success();
        }

        /// <summary>
        /// Run card sale. If successfull, refund entire amount.
        /// </summary>
        public Boolean CardRefund()
        {
            Console.WriteLine("CardRefund()");

            if (!CardSale())
                return false;

            // Refund full amount. Please notice constructor with one argument.
            RefundTransactionRequest rtt = new RefundTransactionRequest(this.TransactionID);

            return
                Refund(rtt).Success();
        }

        /// <summary>
        /// Run check sale. If succcesfull, refund entire amount. In this test we illustrate
        /// use of overloaded constructor.
        /// </summary>
        public Boolean CheckRefund()
        {
            Console.WriteLine("CheckRefund()");

            if (!CheckSale())
                return false;

            RefundTransactionRequest rtt = 
                new RefundTransactionRequest(this.TransactionID, this.TransactionAmount.Value);

            return
                Refund(rtt).Success();
        }

        /// <summary>
        /// Authorize card.
        /// </summary>
        /// <returns></returns>
        public Boolean AuthorizeNotOnFileCard()
        {
            Console.WriteLine("AuthorizeNotOnFileCard()");
            return
                AuthorizeCard(TransactionsHelper.SampleCardTransaction())
                    .Success();
        }

        /// <summary>
        /// Use previously stored card in new authorization request.
        /// For example, you need to ensure the card on recurring payment schedule
        /// has sufficient funds.
        /// </summary>
        /// <returns></returns>
        public Boolean AuthorizeCardOnFile()
        {
            Console.WriteLine("AuthorizeCardOnFile()");

            // Some time ago we executed card sale
            if (!CardSale())
                return false;

            // Instantiate new CardTransactionRequest and set its TokenID
            CardTransactionRequest ctr = TransactionsHelper.SampleCardTransaction();
            ctr.TokenID = this.TransactionID;
            ctr.Amount = new BluefinAmount(150.00);
           
            return
                AuthorizeCard(ctr)
                    .Success();
        }

        /// <summary>
        /// Store card in Bluefin for future use.
        /// </summary>
        /// <returns></returns>
        public Boolean StoreCard()
        {
            Console.WriteLine("StoreCard()");

            return
                StoreCard(TransactionsHelper.SampleCardTransaction())
                    .Success();
        }

        /// <summary>
        /// When card is stored in Bluefin, you can edit card: 
        /// specify new expiration date, cvv, address etc. 
        /// </summary>
        /// <returns></returns>
        public Boolean UpdateStoredCard()
        {
            Console.WriteLine("UpdateStoredCard()");
            // Let's authorize card not on file
            if (!AuthorizeNotOnFileCard())
                return false;

            // Now let's change some values
            // Set TokenID.
            CardTransactionRequest ctr = TransactionsHelper.SampleCardTransaction();
            ctr.CardExpiration = "0915";
            ctr.Description    = "Updated by UpdateStoredCard()";
            ctr.TokenID        = this.TransactionID;

            return
                StoreCard(ctr).Success();
        }

        /// <summary>
        /// Illustrates all steps to take running Safe Communications Sale
        /// </summary>
        public Boolean SafeCommunicationCardSale()
        {
            Console.WriteLine("SafeCommunicationCardSale()");
            // Initiate transaction 1st.
            if(!InitiateTransaction())
                return false;

            // Use returned transaction id in your sale request
            CardTransactionRequest ctr =
                TransactionsHelper.SampleCardTransaction();

            ctr.TransactionID = TransactionID;

            Boolean success = CardSale(ctr).Success();

            // if sale request failed we can:
            //   - check transaction status. If successfull, transaction was processed fine.
            //   - or re-submit the same transaction. If transaction was processed fine,
            //     Bluefin will reply with DUPLICATE TRANSACTION error message

            // Just to illustrate use of status
            TransactionStatusResponse tsr = TransactionStatus(ctr.TransactionID);

            // Just to illustrate use of status with ISV acct
            String accountID = "000000000001"; // demo acct
            String accessKey = "abcdef123456";

            BluefinAccount isvAccount = new BluefinAccount(accountID, accessKey);
            TransactionStatusResponse tsr2 = TransactionStatus_ISV(ctr.TransactionID, isvAccount);

            return
                tsr.Success() && tsr.Found && tsr.Approved; //if true, card sale is processed.
        }

        public Boolean SafeCommunicationCheckSale()
        {
            Console.WriteLine("SafeCommunicationCheckSale()");
            Console.WriteLine("SafeCommunicationCardSale()");
            // Initiate transaction 1st.
            if (!InitiateTransaction())
                return false;

            // Use returned transaction id in your sale request
            CheckTransactionRequest ctr =
                TransactionsHelper.SampleCheckTransaction();

            ctr.TransactionID = TransactionID;

            Boolean success = CheckSale(ctr).Success();

            // if sale request failed we can:
            //   - check transaction status. If successfull, transaction was processed fine.
            //   - or re-submit the same transaction. If transaction was processed fine,
            //     Bluefin will reply with DUPLICATE TRANSACTION error message

            // Just to illustrate re-submitting the same request
            CheckTransactionResponse tsr = CheckSale(ctr);

            return
                !tsr.Success() && tsr.ErrorCode == 30003;
        }

        /// <summary>
        /// Use it in Safe Communications mode
        /// </summary>
        /// <returns></returns>
        public Boolean InitiateTransaction()
        {
            Console.WriteLine("InitiateTransaction()");
            Boolean success = true;

            // ISV:
            //BluefinAccount acct = new BluefinAccount("000000000001", "abcdef123456");
            //InitiateTransactionResponse itr =
            //    BluefinGateway.Transaction.Initiate(acct);

            InitiateTransactionResponse itr =
                BluefinGateway.Transaction.Initiate();

            success = itr.Success();

            if (success)
            {
                TransactionID = itr.TransactionID;
                Console.WriteLine("Initiated transaction id {0}", itr.TransactionID);
            }
            else
            {
                Console.WriteLine("Failed initiating transaction: ErrorCode={0}, ErrorMessage={1}", itr.ErrorCode, itr.ErrorMessage);
            }
            return success;
        }

        /// <summary>
        /// Runs Sale(). If successfull, TransactionID is set.
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public CardTransactionResponse CardSale(CardTransactionRequest request)
        {
            // Reset class variables that other tests may depends on
            TransactionID     = null; 
            TransactionAmount = null;

            Console.WriteLine("Card Sale Request ..." 
                + LogHelper.ToLogString(request, "\n\t"));

            CardTransactionResponse response =
                BluefinGateway.Transaction.Sale(request);

            // Store response values for future tests that may follow
            this.TransactionID     = response.TransactionID;
            this.TransactionAmount = response.Amount;
 
            Console.WriteLine("Card Sale Response ..."
                + LogHelper.ToLogString(response, "\n\t"));

            return response;
        }

        public CheckTransactionResponse CheckSale(CheckTransactionRequest request)
        {
            // Reset class variables that other tests may depends on
            TransactionID = null;
            TransactionAmount = null;

            Console.WriteLine("Check Sale Request ..."
                + LogHelper.ToLogString(request, "\n\t"));

            CheckTransactionResponse response = 
                BluefinGateway.Transaction.Sale(request);

            Console.WriteLine("Check Sale Response ..."
                + LogHelper.ToLogString(response, "\n\t"));

            // Store response values for other tests that may follow
            this.TransactionID     = response.TransactionID;
            this.TransactionAmount = response.Amount;

            return response;
        }

        public TransactionResponse SwipedCardSale(SwipedCardTransactionRequest request)
        {
            // Reset class variables that other tests may depends on
            TransactionID = null;
            TransactionAmount = null;

            Console.WriteLine("Swiped Card Sale Request ..."
                + LogHelper.ToLogString(request, "\n\t"));

            CardTransactionResponse response =
                BluefinGateway.Transaction.Sale(request);

            Console.WriteLine("Swiped Card Sale Response ..."
                + LogHelper.ToLogString(response, "\n\t"));

            // Store response values for other tests that may follow
            this.TransactionID = response.TransactionID;
            this.TransactionAmount = response.Amount;

            return response;
        }

        public CardTransactionResponse StoreCard(CardTransactionRequest request)
        {
            // Reset class variables that other tests may depends on
            TransactionID = null;
            TransactionAmount = null;

            Console.WriteLine("Store Card Request ..."
                + LogHelper.ToLogString(request, "\n\t"));

            CardTransactionResponse response =
                BluefinGateway.Transaction.Store(request);

            Console.WriteLine("Store Card Response ..."
                + LogHelper.ToLogString(response, "\n\t"));

            // Store response values for other tests that may follow
            this.TransactionID = response.TransactionID;
            this.TransactionAmount = response.Amount;

            return response;
        }

        public TransactionResponse Capture(CaptureTransactionRequest request)
        {
            Console.WriteLine("Capture Request ..."
                + LogHelper.ToLogString(request, "\n\t"));

            // Please notice, that Capture() returns:
            //  instance of TransactionResponse,      if tr.TenderType == null            or
            //  instance of CardTransactionResponse,  if tr.TenderType == TenderType.CARD or
            //  instance of CheckTransactionResponse, if tr.TenderType == TenderType.ACH
            TransactionResponse response =
                BluefinGateway.Transaction.Capture(request);

            // Store it for other tests
            this.TransactionTenderType = response.TenderType;

            // Cast returned object, if desired
            if (response.TenderType == TenderType.CARD)
            {
                Console.WriteLine("Card Response ..."
                    + LogHelper.ToLogString((CardTransactionResponse)response, "\n\t"));
            }
            else if (response.TenderType == TenderType.ACH)
            {
                Console.WriteLine("Check Response ..."
                    + LogHelper.ToLogString((CheckTransactionResponse)response, "\n\t"));
            }
            else
            {
                Console.WriteLine("No tender Response ..."
                    + LogHelper.ToLogString(response, "\n\t"));
            }

            return response;
        }

        public TransactionResponse Refund(RefundTransactionRequest request)
        {
            Console.WriteLine("Refund Request ..."
                + LogHelper.ToLogString(request, "\n\t"));

            // Please notice, that Capture() returns:
            //  instance of TransactionResponse,      if tr.TenderType == null            or
            //  instance of CardTransactionResponse,  if tr.TenderType == TenderType.CARD or
            //  instance of CheckTransactionResponse, if tr.TenderType == TenderType.ACH
            TransactionResponse response =
                BluefinGateway.Transaction.Refund(request);

            // Store it for other tests
            this.TransactionTenderType = response.TenderType;

            // Cast returned object, if desired
            if (response.TenderType == TenderType.CARD)
            {
                Console.WriteLine("Card Response ..."
                    + LogHelper.ToLogString((CardTransactionResponse)response, "\n\t"));
            }
            else if (response.TenderType == TenderType.ACH)
            {
                Console.WriteLine("Check Response ..."
                    + LogHelper.ToLogString((CheckTransactionResponse)response, "\n\t"));
            }
            else
            {
                Console.WriteLine("No tender Response ..."
                    + LogHelper.ToLogString(response, "\n\t"));
            }

            return response;
        }

        public CardTransactionResponse AuthorizeCard(CardTransactionRequest request)
        {
            Console.WriteLine("Authorize Card Request ..."
                + LogHelper.ToLogString(request, "\n\t"));
            
            CardTransactionResponse response = 
                BluefinGateway.Transaction.Authorize(request);

            // Store TransactionID for other test
            this.TransactionID = response.TransactionID;

            Console.WriteLine("Card Response ..."
                + LogHelper.ToLogString(response, "\n\t"));

            return response;
        }

        public TransactionStatusResponse TransactionStatus(String transactionID)
        {
            Console.WriteLine("Requesting status of transaction id {0}", transactionID);

            TransactionStatusResponse response = BluefinGateway.Transaction.Status(transactionID);

            Console.WriteLine("Transaction Status Response ..."
                + LogHelper.ToLogString(response, "\n\t"));

            return response;
        }

        public TransactionStatusResponse TransactionStatus_ISV(String transactionID, BluefinAccount isvAcct)
        {
            Console.WriteLine("Requesting status of transaction id {0} for account_id {1} with api_accesskey {2}", transactionID, isvAcct.AccountID, isvAcct.AccessKey);

            TransactionStatusResponse response = BluefinGateway.Transaction.Status(transactionID, isvAcct);

            Console.WriteLine("Transaction Status Response ..."
                + LogHelper.ToLogString(response, "\n\t"));

            return response;
        }
    }
}
